/********************************************************************************************************
* PROGRAM      : fgPlugins
* DATE - TIME  : mardi 25 juillet 2006 - 18h57
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox@gmail.com )
* FILENAME     : fgDialog.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "fgDialog.h"
//
#include <QMouseEvent>
#include <QBitmap>
#include <QLabel>
#include <QListWidget>
//
fgDialog::fgDialog( QWidget* parent, Qt::WindowFlags flags )
	: QDialog( parent, flags ), _autoResize( true ), _data( "" ), _movable( true )
{
	setAttribute( Qt::WA_DeleteOnClose );
	// set default dialog font
	QFont f = font();
#if defined Q_OS_WIN
	f.setPixelSize( 11 );
#elif defined Q_OS_MAC
	f.setPixelSize( 10 );
#else
	f.setPixelSize( 10 );
#endif
	setFont( f );
}
//
void fgDialog::mouseMoveEvent( QMouseEvent* e )
{
	if ( _movable )
		move( e->globalPos() -mMouse );
	QDialog::mouseMoveEvent ( e );
}
//
void fgDialog::mousePressEvent ( QMouseEvent* e )
{
	if ( _movable )
	{
		setMouse( e->pos() );
	}
	QDialog::mousePressEvent( e );
}
//
QPoint fgDialog::mouse() const
{
	return mMouse;
}
//
void fgDialog::fixMacWidgets( QWidget* widget )
{
#if defined Q_OS_MAC
	foreach ( QLabel* label, widget->findChildren<QLabel*>() )
		label->setFont( widget->font() );
	foreach ( QListWidget* lw, widget->findChildren<QListWidget*>() )
		lw->setFont( widget->font() );
	foreach ( QWidget* w, widget->findChildren<QWidget*>() )
		w->setAttribute( Qt::WA_MacShowFocusRect, false );
#else
	Q_UNUSED( widget )
#endif
}
//
void fgDialog::fixMacWidgets()
{
	fixMacWidgets( this );
}
//
void fgDialog::setMouse( const QPoint& p)
{
	if ( mMouse != p )
		mMouse = p;
}
//
bool fgDialog::autoResize() const
{
	return _autoResize;
}
//
void fgDialog::setAutoResize( const bool b )
{
	if ( _autoResize != b )
		_autoResize = b;
}
//
QVariant fgDialog::data() const
{
	return _data;
}
//
void fgDialog::setData( const QVariant& v )
{
	if ( _data != v )
	{
		_data = v;
		emit dataChanged();
		emit dataChanged( _data );
		emit dataChanged( _data.toString() );
		emit dataChanged( _data.toInt() );
	}
}
//
bool fgDialog::movable() const
{
	return _movable;
}
//
void fgDialog::setMovable( const bool b )
{
	if ( _movable != b )
		_movable = b;
}
//
QPixmap fgDialog::backgroundPixmap() const
{
	return _backgroundPixmap;
}
//
void fgDialog::setBackgroundPixmap( const QPixmap& p )
{
	if ( _backgroundPixmap.toImage() != p.toImage() )
	{
		_backgroundPixmap = p;
		// Affiche le pixmap
		QPalette palette( this->palette() );
		palette.setBrush( backgroundRole(), QBrush( p ) );
		setPalette( palette );
		// Si le pixmap n'est pas nul et qu'il faut redimensionn� on le fait avec la taille du pixmap
		if ( !p.isNull() && _autoResize && size() != p.size() )
			resize( p.size() );
		// set dialog transparent
		setMask( p.mask() );
	}
}
//
void fgDialog::setWidgetTransparent( QWidget* w, const QPixmap& p, bool b, QPalette::ColorRole r )
{
	QPalette n = w->palette();
	n.setBrush( r, QBrush( b ? p.copy( w->geometry() ) : p ) );
	w->setPalette( n );
}
