// VLC Recorder
// FreeGO - Recorder.cpp
#include <QApplication>

#include "Recorder.h"
#include "src/misc.h"
#include "src/ui/UIConfigure.h"
#include "src/Options.h"

#include <QDebug>

QPointer<Recorder> Recorder::pSelf = 0L;

Recorder* Recorder::self( QWidget* inParent )
{
	if ( !pSelf )
	{
		pSelf = new Recorder( inParent );
	}
	return pSelf;
}

// ================
// ### RECORDER ###
// ================
Recorder::Recorder( QWidget* inParent )
{
	parent 	= inParent;
	init	= true;
	
	// TCP Handler
	socket 	= new QTcpSocket;

	// Timer : we're checking records each minute
	timer 	= new QTimer(this);

	// Connections
	connect(timer, 	SIGNAL(timeout()), this, SLOT(checkRecords()));
	
	// Grabbing Records From Ini File
	loadRecords();
	
	// start check
	timer->start(1000*20); //60000
}

// ==================
// ### ADD RECORD ###
// ==================
void Recorder::addRecord( Record *inRec, bool modif )
{
	// STEP 0 - Port Init (9000 to 10000)
	inRec->port = BASE_PORT + (lastPortUsed%1000);
	lastPortUsed++;
	
	
	// STEP 1 - Time Check
	if (inRec->hstart >= inRec->hstop)
	{
		//UIMAIN->alert(QString::fromUtf8("Votre programmation horaire est incorrecte !\n Votre heure de démarrage est supérieure ou égale à votre heure d'arrêt"));
		//return;
	}

	
	// STEP 2 - Date Check
	for (int i=0; i<records.size(); i++)
	{
		if ( records.at(i)->channel == inRec->channel )
			if ( (( inRec->hstart >= records.at(i)->hstart ) && ( inRec->hstart <= records.at(i)->hstop )) ||
                                        (( inRec->hstop >= records.at(i)->hstart) && inRec->hstop <= records.at(i)->hstop ))
				{
				UIMAIN->alert(QString::fromUtf8("Votre programmation est en conflit avec une autre déjà définie."));
				return;
			}
	}

	inRec->enabled = false;
	records << inRec;

	if (!init)
	{
		if(modif)
			UIMAIN->alert( QString::fromUtf8("Modification effectuée"), true, true );
		else
			UIMAIN->alert( QString::fromUtf8("Enregistrement ajouté à la liste"), true, true );
		
		// Sauvegarde dans FreeGo.ini
		Options::current()->beginGroup( QString("Recorder/Rec_%1").arg( records.count()-1 ) );

		Options::current()->setValue( "channel",	inRec->channel);
		Options::current()->setValue( "url",		inRec->url);
		Options::current()->setValue( "path",		inRec->path);
		Options::current()->setValue( "hstart",		inRec->hstart.toString("hh:mm"));
		Options::current()->setValue( "hstop",		inRec->hstop.toString("hh:mm"));
		Options::current()->setValue( "weekly", 	inRec->weekly);
		
		if ( !inRec->date.isNull() )
			Options::current()->setValue( "date",	inRec->date.toString("ddMMyyyy"));
		
		if ( !inRec->days.isEmpty() )
			Options::current()->setValue( "days",	inRec->days);
		
		Options::current()->endGroup();
	}
	
	emit newRecord(getLastRecord());
}

// ==================
// ### GET RECORD ###
// ==================
Record* Recorder::getRecord(int index)
{
	return records.at(index);
}

// =======================
// ### GET LAST RECORD ###
// =======================
Record* Recorder::getLastRecord()
{
	if (!records.isEmpty())
		return records.last();
	else
		return NULL;
}

// =====================
// ### GET NBRECORDS ###
// =====================
int Recorder::getNbRecords()
{
	return records.size();
}

// =====================
// ### CHECK RECORDS ###
// =====================
void Recorder::checkRecords()
{
	QString num;
	
	if ( records.size() == 0 )
		return;

	// 1st Pass - Start/Stop
	for ( int i = 0; i < records.count(); i++ )
	{
		Record* rec = records[i];
		const QDateTime dtCurrent = QDateTime::currentDateTime();
		// STEP 1 - Date Check
		if ( ( rec->weekly && rec->days.contains( num.setNum( dtCurrent.date().dayOfWeek() ) ) ) || ( !rec->weekly && rec->date == dtCurrent.date() ) )
		{
			// STEP 2 - Time Check
			if ( ( !rec->enabled ) && ( rec->hstart.hour() == dtCurrent.time().hour() ) && ( rec->hstart.minute() == dtCurrent.time().minute() ) )		 
			{	
				QString command = QString( "\"%1\" --intf rc --rc-host \"127.0.0.1:%2\" --sout file/ts:\"%3\" \"%4\"" )
				.arg( Options::current()->value( "/FreeGo/Options/VLCpath" ).toString() )
				.arg( rec->port )
				.arg( rec->path )
				.arg( rec->url ); // Channel URL
				vlc.start( command );
				
				// Updating State
				rec->enabled = true;
				rec->state = "En cours...";
				emit changeState( i, 1 );
			}
			else if ( ( rec->enabled ) && ( rec->hstop.hour() == dtCurrent.time().hour() ) && ( rec->hstop.minute() == dtCurrent.time().minute() ) )
			{
				// Sending Shutdown order to VLC
				socket->connectToHost( "127.0.0.1", rec->port );
				socket->waitForConnected( 500 );
				socket->write( "\n" );
				socket->write( "quit\n" );
				socket->close();
				rec->enabled = false;
				rec->state = QString::fromUtf8( "Terminé" );
				emit changeState( i, 2 );
				emit recordFinished( rec );
			}
		}
	}
}

// =====================
// ### REMOVE RECORD ###
// =====================
void Recorder::removeRecord(int idx)
{
	// STEP 1 - Removing Record
	records.removeAt(idx);

	// STEP 2 - Re-writing INI File
	saveRecords();
}

// ====================
// ### SAVE RECORDS ###
// ====================
void Recorder::saveRecords()
{
	closeSocket = new QTcpSocket;
	
	Options::current()->beginGroup("Recorder");
	Options::current()->remove("");
	
	// Saving Pending Records
	for(int i=0; i<records.count(); i++)
	{
		// Sending Shutdown order to VLC
		if (records.at(i)->enabled)
		{
			closeSocket->connectToHost("127.0.0.1", records.at(i)->port);
			closeSocket->waitForConnected(500);
			closeSocket->write("\n");
			closeSocket->write("quit\n");
			closeSocket->waitForReadyRead(500);
			closeSocket->close();
		}
		
		if ( (!records.at(i)->enabled) && (records.at(i)->state != QString::fromUtf8("Terminé")) )
		{
			Options::current()->beginGroup( QString("Rec_%1").arg(i) );
			
			Options::current()->setValue( "channel",	records.at(i)->channel);
			Options::current()->setValue( "url", 		records.at(i)->url);
			Options::current()->setValue( "hstart", 	records.at(i)->hstart.toString("hh:mm"));
			Options::current()->setValue( "hstop", 		records.at(i)->hstop.toString("hh:mm"));
			Options::current()->setValue( "date",		records.at(i)->date.toString("ddMMyyyy"));
			Options::current()->setValue( "path",		records.at(i)->path);
			Options::current()->setValue( "days",		records.at(i)->days);
			Options::current()->setValue( "weekly", 	records.at(i)->weekly);
		
			Options::current()->endGroup();
		}
	}
	Options::current()->endGroup();	
}

// ====================
// ### LOAD RECORDS ###
// ====================
void Recorder::loadRecords()
{
	Options::current()->beginGroup("Recorder");
	QStringList keys = Options::current()->childGroups();
	QStringList recordline;
	
	for (int i=0; i<keys.count(); i++)
	{
		Options::current()->beginGroup(keys.at(i));
		
		Record *record 	= new Record;
		
		record->channel	= Options::current()->value("channel").toString();
		record->url		= Options::current()->value("url").toString();
		record->path	= Options::current()->value("path").toString();
		record->hstart	= QTime::fromString( Options::current()->value("hstart").toString(), 	"hh:mm" );
		record->hstop	= QTime::fromString( Options::current()->value("hstop").toString(),		"hh:mm" );
		record->weekly	= Options::current()->value("weekly").toInt();
		
		if (!Options::current()->value("date").isNull())
			record->date = QDate::fromString( Options::current()->value("date").toString(),		"ddMMyyyy" );
			
		if (!Options::current()->value("days").isNull())
			record->days = Options::current()->value("days").toString();

		Options::current()->endGroup();
		
		addRecord( record );
	}
	
	init = false;

	// Removing Group in Ini File
	Options::current()->endGroup();	
}
