/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : mardi 01 aot 2006 - 21h42
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : ThreadFreeGoInformations.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "ThreadFreeGoInformations.h"
#include "misc.h"
#include "main.h"
#include "Options.h"

#include <QSettings>
#include <QUrl>

#include <QDebug>

QPointer<ThreadFreeGoInformations> ThreadFreeGoInformations::mSelf;

ThreadFreeGoInformations* ThreadFreeGoInformations::self()
{
	if ( !mSelf )
		mSelf = new ThreadFreeGoInformations;
	return mSelf;
}

ThreadFreeGoInformations::ThreadFreeGoInformations()
{
	mPubligoShown = false;
	connect( this, SIGNAL( requestInformations() ), this, SLOT( requestInformationsFile() ) );
	connect( this, SIGNAL( requestMessages() ), this, SLOT( requestMessagesFile() ) );
}

void ThreadFreeGoInformations::run()
{
	// first check for publigo
	emit requestInformations();
	forever
	{
		if ( mPubligoShown )
		{
			emit requestInformations();
			emit requestMessages();
#ifdef QT_NO_DEBUG
			// Attend 5min avant de recommencer si publigo a deja ete affiche
			sleep( 300 );
#else
			// En mode debug: attend 10s avant de recommencer
			sleep( 10 );
#endif
		}
		else
			sleep( 10 );
	}
}

bool ThreadFreeGoInformations::publigoShown() const
{
	return mPubligoShown;
}

void ThreadFreeGoInformations::setPubligoShow()
{
	mPubligoShown = true;
}

void ThreadFreeGoInformations::requestInformationsFile()
{
	QUrl mUrl = QUrl( REQUESTER->freeGoBaseUrl().append( "notif/index.php" ) );
	REQUESTER->requestUrl( this, "readFreeGoInformations", mUrl );
}

void ThreadFreeGoInformations::readFreeGoInformations( QSettings* s, bool error )
{
	if ( error )
		return;

	uint validityMax = 60 *60 *24; // A day
	QDateTime optionsVersion;
	uint now = QDateTime::currentDateTime().toTime_t();
	uint validity;
	QDateTime version;
	QString message, url;
	QString dateTimeMask = "yyyy-MM-dd HH:mm";

	if ( mPubligoShown )
	{
		// MAJFREEGO
		if ( s->value( "MAJFREEGO/Version" ).toString() > PROGRAM_VERSION && s->value( "MAJFREEGO/Version" ).toString() > Options::current()->value( "Version/FreeGo" ).toString() )
		{
			url = s->value( "MAJFREEGO/Url" ).toString();
			if ( Options::current()->value( "FreeGo/Notifications/FreeGo", false ).toBool() )
				emit programUpdate( -1, QDateTime::currentDateTime(), s->value( "MAJFREEGO/Message" ).toString().arg( url ) );
			else
				emit programUpdating( -1, QDateTime::currentDateTime(), url );
		}
		Options::current()->setValue( "Version/FreeGo", s->value( "MAJFREEGO/Version" ).toString() );

		// MAJFREEBOX
		optionsVersion = Options::current()->value( "Version/FreeBox" ).toDateTime();
		validity = s->value( "MAJFREEBOX/Validite" ).toInt();
		version = QDateTime::fromString( s->value( "MAJFREEBOX/Version" ).toString(), dateTimeMask );
		message = s->value( "MAJFREEBOX/Message" ).toString();
		if ( Options::current()->value( "FreeGo/Notifications/FreeBox", true ).toBool() && version > optionsVersion && now -version.toTime_t() <= validityMax *validity )
			emit freeboxUpdate( validity, version, message );
		Options::current()->setValue( "Version/FreeBox", version );

		// ACTUFREE
		optionsVersion = Options::current()->value( "Version/Actuality" ).toDateTime();
		validity = s->value( "ACTUFREE/Validite" ).toInt();
		version = QDateTime::fromString( s->value( "ACTUFREE/Version" ).toString(), dateTimeMask );
		message = s->value( "ACTUFREE/Message" ).toString();
		if ( Options::current()->value( "FreeGo/Notifications/Actuality", true ).toBool() && version > optionsVersion && now -version.toTime_t() <= validityMax *validity )
			emit freeActualityUpdate( validity, version, message );
		Options::current()->setValue( "Version/Actuality", version );

		// MSGEQUIPE
		optionsVersion = Options::current()->value( "Version/Message" ).toDateTime();
		validity = s->value( "MSGEQUIPE/Validite" ).toInt();
		version = QDateTime::fromString( s->value( "MSGEQUIPE/Version" ).toString(), dateTimeMask );
		message = s->value( "MSGEQUIPE/Message" ).toString();
		if ( Options::current()->value( "FreeGo/Notifications/Message", true ).toBool() && version > optionsVersion && now -version.toTime_t() <= validityMax *validity )
			emit teamMessageUpdate( validity, version, message );
		Options::current()->setValue( "Version/Message", version );
	}
	else
	{
		// ANNONCE
		optionsVersion = Options::current()->value( "Version/Annonce" ).toDateTime();
		validity = s->value( "ANNONCE/Validite" ).toInt();
		version = QDateTime::fromString( s->value( "ANNONCE/Version" ).toString(), dateTimeMask );
		message = s->value( "ANNONCE/Message" ).toString();
		int repeat = s->value( "ANNONCE/Repeat" ).toInt();
		if ( (version > optionsVersion && now -version.toTime_t() <= validityMax *validity) || repeat == 1 )
			emit freegoAnnonce( validity, version, message, s->value( "ANNONCE/Taille" ).toString() );
		else
			mPubligoShown = true;
		Options::current()->setValue( "Version/Annonce", version );
	}
}

void ThreadFreeGoInformations::requestMessagesFile()
{
	if ( Options::current()->value( "FreeGo/Notifications/Repondeur", true ).toBool() )
		REQUESTER->login( this, "readLogin" );
}

void ThreadFreeGoInformations::readLogin( QSettings*, bool error )
{
	if ( error )
	{
		emit alert( QString::fromUtf8("Impossible de vous identifier sur free, vérification du répondeur impossible.") );
		return;
	}

	REQUESTER->requestUrl( this, "readMessages", REQUESTER->freeBaseUrl().append( "notification_tel.php" ), "GET", pHttpRequest::rtByteArray );
}

void ThreadFreeGoInformations::readMessages( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		emit alert( QString::fromUtf8("Impossible de vérifier votre repondeur.") );
		return;
	}
	
	// parse
	QString ba = QString::fromLatin1( buffer );
	
	QString key = "<div  id=\"content\" class=\"telephone\">";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos );
	
	key = "<div class=\"clearer\">";
	pos = ba.indexOf( key );
	ba = ba.left( pos ).simplified();
	
	// parse messages
	QRegExp rx( "<table.*>.*</table>" );
	rx.setMinimal( true );
	if ( rx.indexIn( ba ) != -1 )
	{
		// get table tag
		QString table = rx.cap( 0 ).simplified();
		if ( table.contains( "Pas de nouveau message", Qt::CaseInsensitive ) )
			emit newMessages( 0 );
		else
		{
			// parse table
			QList<QStringList> mTable = misc::parseHtmlTable( table, false );
			emit newMessages( mTable.count() );
		}
	}
}
