/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : mardi 01 aout 2006 - 18h30
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : ThreadMailChecker.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "ThreadMailChecker.h"
#include "Options.h"
#include "misc.h"
#include "main.h"
#include "ThreadFreeGoInformations.h"

#include <QHostAddress>
#include <QTcpSocket>
#include <QSettings>

const QByteArray imapEol = "\r\n";

QPointer<ThreadMailChecker> ThreadMailChecker::mSelf;

ThreadMailChecker* ThreadMailChecker::self()
{
	if ( !mSelf )
		mSelf = new ThreadMailChecker;
	return mSelf;
}

void ThreadMailChecker::run()
{
	mSocket = new QTcpSocket();
	int n;
	EmailAccount* a;

	forever
	{
		if ( ThreadFreeGoInformations::self()->publigoShown() )
		{
			if ( Options::current()->value( "FreeGo/Notifications/Emails", true ).toBool() )
			{
				// read quickly all email accounts as QSettings is not thread safe
				qDeleteAll( accounts );
				accounts.clear();
				int s = Options::current()->beginReadArray( "MailAccounts" );
				for ( int i = 0; i < s; i++ )
				{
					Options::current()->setArrayIndex( i );
					a = new EmailAccount;
					a->Login = Options::current()->value( "UserName" ).toString();
					a->Password = misc::decrypt( Options::current()->value( "Password" ).toString() );
					a->Server = Options::current()->value( "Server" ).toString();
					accounts << a;
				}
				Options::current()->endArray();

				// check accounts
				foreach ( EmailAccount* a, accounts )
				{
					n = checkInbox( a->Login, a->Password );
					if ( n > 0 )
						emit newEmails( a->Login, n );
				}
			}
#ifdef QT_NO_DEBUG
			// Attend 5min avant de recommencer
			sleep( 300 );
#else
			// En mode debug: attend 10s avant de recommencer
			sleep( 10 );
#endif
		}
		else
			sleep( 10 );
	}
}

void ThreadMailChecker::closeSocket()
{
	if ( mSocket )
		mSocket->close();
}

bool ThreadMailChecker::writeToSocket( const QString& s )
{
	QByteArray a = s.toLocal8Bit() +imapEol;
	qint64 w = 0;
	while ( w < (qint64)a.size() )
	{
		qint64 r = mSocket->write( a.data() + w, a.size() -w );
		if ( r == -1 )
			return false;
		w += r;
	}
	return mSocket->waitForBytesWritten();
}

bool ThreadMailChecker::readLineFromSocket( QByteArray& a )
{
	while ( !mSocket->canReadLine() )
	{
		if ( !mSocket->waitForReadyRead( 30 *1000 ) )
		{
			closeSocket();
			return false;
		}
	}

	qint64 length = mSocket->readLine( a.data(), a.size() );
	if ( length == -1 )
	{
		closeSocket();
		return false;
	}
	return true;
}

int ThreadMailChecker::checkInbox( const QString& l, const QString& p )
{
	const int i = 256;
	int n = 0;
	QByteArray a;
	a.resize( i );

	// Connect To IMAP Server
	closeSocket();
	mSocket->connectToHost( "imap.free.fr", 143 );
	if ( !mSocket->waitForConnected( 5000 ) ) // 5 seconds
	{
		emit alert( "Impossible de contacter le serveur de messagerie" );
		return 0;
	}

	// Read OK Response
	if ( !readLineFromSocket( a ) )
	{
		emit alert( "Read OK Response: Error" );
		return 0;
	}

	// Login TO IMAP Server
	writeToSocket( QString( "a001 LOGIN %1 %2" ).arg( l.left( l.indexOf( "@" ) ), p ) );

	// Read Response
	if ( !readLineFromSocket( a ) )
	{
		emit alert( QString::fromUtf8("Vérification messagerie: erreur de communication avec le serveur") );
		return 0;
	}

	// Login OK
	if ( a.indexOf( "OK" ) == -1 )
	{
		closeSocket();
		emit alert( QString::fromUtf8( "Vérification messagerie: Echec à l'identification pour %1" ).arg( l ) );
		return 0;
	}

	// Select INBOX
	writeToSocket( "a002 SELECT INBOX" );

	// Read all data
	bool b = false;
	while ( !b )
	{
		if ( !readLineFromSocket( a ) )
			return 0;
		if ( a.startsWith( "a002" ) )
			b = true;
	}

	// Look for unseen message
	writeToSocket( "a003 SEARCH UNSEEN" );

	// Count unread message
	if ( !readLineFromSocket( a ) ) 
	{
		emit alert( QString::fromUtf8( "Vérification messagerie: Impossible de compter les emails non lus pour %1" ).arg( l ) );
		return 0;
	}

	// The line is like "* SEARCH 1 2 3 4 5 6"
	QStringList list = QString::fromLatin1( a ).trimmed().split( ' ' );
	n = list.count() -2;

	// Close connection
	writeToSocket( "a004 LOGOUT" );
	closeSocket();

	return n;
}
