#include "XmlReader.h"

#include <QStringList>

QString XmlReader::fixedPath( const QString& _path )
{
	QString path = _path;
	
	path.replace( "\\", "/" ).simplified().remove( " " );
	
	while ( path.startsWith( "/" ) )
	{
		path.remove( 0, 1 );
	}
	
	while ( path.contains( "//" ) )
	{
		path.replace( "//", "/" );
	}
	
	if ( path.endsWith( "/" ) )
	{
		path.chop( 1 );
	}
	
	return path;
}

QString XmlReader::readPath( const QDomDocument& document, const QString& _path, const QString& _attr )
{
	const QString path = fixedPath( _path );
	const QStringList parts = path.split( "/" );
	const QDomElement rootElement = document.documentElement();
	QDomNode node = rootElement;
	
	// check root tag match first part
	if ( rootElement.tagName().toLower() != parts.value( 0 ).toLower() )
	{
		return QString::null;
	}
	
	// walk around document
	for ( int i = 1; i < parts.count(); i++ )
	{
		const QString part = parts.at( i );
		QDomNode child = node.firstChildElement( part );
		
		if ( child.isNull() )
		{
			child = node.toElement().attributes().namedItem( part );
		}
		
		if ( child.isNull() )
		{
			return QString::null;
		}
		
		node = child;
	}
	
	if(_attr != QString::null)
	{
		return node.toElement().attribute( _attr );
	}
	else
		return node.nodeValue().isEmpty() ? node.firstChild().toText().data() : node.nodeValue();
}
