/********************************************************************************************************
	* PROGRAM      : FreeGo4
	* DATE - TIME  : mardi 25 juillet 2006 - 22h56
	* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox gmail.com )
	* FILENAME     : pHTTPRequest.cpp
	* LICENSE      : GPL
	* COMMENTARY   : 
	********************************************************************************************************/
#include "pHttpRequest.h"
#include "main.h"
#include "misc.h"
#include "XmlReader.h"

using namespace misc;
//
#include <QSettings>
#include <QDomDocument>
//
QPointer<pHttpRequest> pHttpRequest::mSelf = 0L;
//
pHttpRequest* pHttpRequest::self( QObject* p )
{
	if ( !mSelf )
		mSelf = new pHttpRequest( p );
	return mSelf;
}
//
pHttpRequest::pHttpRequest( QObject* p )
	: QHttp( p )
{
	// the url to log to
	mFreeBaseUrl = "https://iphone.free-go.net/";
	mFreeLoginUrl = QString( "%1identification.php" ).arg( mFreeBaseUrl );
	mFreeGoBaseUrl = "http://www.free-go.net/";
	mFreeSSLBaseUrl = "https://adsls.free.fr/";
	// connect request finished slot to manage them
	connect( this, SIGNAL( requestFinished( int, bool ) ), this, SLOT( requestFinished( int, bool ) ) );
	connect( this, SIGNAL( sslErrors( const QList<QSslError>& ) ), this, SLOT( sslErrors( const QList<QSslError>& ) ) );
}
/*
This member request a file
	- o is the object that send the request
	- s is the slot to call when request finished
	- u is the url file to request
	- m is the request method
	- t is the type method to call from sender o
*/
int pHttpRequest::requestUrl( QObject* o, const char* s, QUrl u, const QString& m, pHttpRequest::RequestType t, const QByteArray& data )
{
	// setting proxy if needed
	Proxy p = proxy();
	if ( p.Enabled )
		setProxy( p.Host, p.Port,  p.Login, p.Password );
	// setting host/port
	const bool isHTTPS = u.scheme().toLower() == "https";
	int port = u.port() != -1 ? u.port() : ( isHTTPS ? 443 : 80 );
	QHttp::ConnectionMode mode = isHTTPS ? QHttp::ConnectionModeHttps : QHttp::ConnectionModeHttp;
	
	setHost( u.host(), mode, port );
	
	//qWarning() << u.host() << mode << port;
	
	// if not login url
	if ( u.toString( QUrl::StripTrailingSlash | QUrl::RemoveQuery | QUrl::RemoveFragment ).toUpper() != freeLoginUrl().toUpper() )
	{
		// if not query, and value not empty
		if ( !u.hasQueryItem( "id" ) && !mId.isEmpty() )
			u.addQueryItem( "id", mId );
		if ( !u.hasQueryItem( "idt" ) && !mIdt.isEmpty() )
			u.addQueryItem( "idt", mIdt );
	}
	
	// creating the http header
	QHttpRequestHeader h( m, u.toString( QUrl::RemoveScheme | QUrl::RemoveAuthority | QUrl::RemoveFragment | QUrl::StripTrailingSlash ), 1, 1 );
	h.setValue( "Host", u.host() );
	
	/*
	if ( !referer.isEmpty() )
		header.setValue( "Referer", referer );
	*/
	h.setContentLength( 0 );
	// create a temp file to get data
	QTemporaryFile* f = new QTemporaryFile( QString( "%1/%2" ).arg( QDir::tempPath(), PROGRAM_NAME ), this );
	f->open();
	// request file and return request id
	int i = request( h, data, f );
	mRequests[i] = pRequest( o, s, u, f, t );
	return i; // for those who wants to get request id
}
//
int pHttpRequest::requestUrl( QUrl url, const QString& method, const QByteArray& data )
{ return requestUrl( 0, 0, url, method, pHttpRequest::rtSettings, data ); }
//
void pHttpRequest::sslErrors( const QList<QSslError>& errors )
{
	Q_UNUSED( errors );
	ignoreSslErrors();
}
//
void pHttpRequest::requestFinished( int i, bool b )
{
	// if ( id is a request done with requestUrl
	if ( mRequests.contains( i ) )
	{
		// look last response
		QHttpResponseHeader h = lastResponse();
		// if request is login request
		if ( i == mIdLogin )
		{
			// clear query value
			mId.clear();
			mIdt.clear();
			// if no error
			if ( !b )
			{
				if ( h.statusCode() == 302 || h.statusCode() == 200 )
				{
					QFile* file = mRequests[ i ].file;
					
					file->seek( 0 );
					
					QDomDocument document( "Free" );
					
					if ( document.setContent( file->readAll() ) )
					{
						mId = XmlReader::readPath( document, "identifiant/id" );
						mIdt = XmlReader::readPath( document, "identifiant/idt" );
					}
					
					file->seek( 0 );
				}
			}
			if ( mId.isEmpty() || mIdt.isEmpty() )
				b = true;
		}
		if ( h.statusCode() == 404 )
			b = true;
		
		// getting the request i
		pRequest p = mRequests[i];
		// go back to file begin
		p.file->seek( 0 );
		// emit signal
		emit customRequestFinished( i, b, p.file );
		// go back to file begin
		p.file->seek( 0 );
		//
		switch ( p.type )
		{
		case rtSettings:
			{
				// creating default settings
				QSettings* settings = new QSettings( p.file->fileName(), QSettings::IniFormat );
				// invoking the request object slot
				if ( p.sender && p.slot )
					QMetaObject::invokeMethod( p.sender, p.slot, Q_ARG( QSettings*, settings ), Q_ARG( bool, b ) );
				// free memory
				delete settings;
				break;
			}
		case rtFile:
			{
				if ( p.sender && p.slot )
					QMetaObject::invokeMethod( p.sender, p.slot, Q_ARG( QString, p.file->fileName() ), Q_ARG( bool, b ) );
				break;
			}
		case rtByteArray:
			{
				if ( p.sender && p.slot )
					QMetaObject::invokeMethod( p.sender, p.slot, Q_ARG( QByteArray, p.file->readAll() ), Q_ARG( bool, b ) );
				break;
			}
		case rtDomDocument:
			{
				if ( p.sender && p.slot )
				{
					QDomDocument document( "Free" );
					document.setContent( QString::fromLatin1( p.file->readAll() ) );
					QMetaObject::invokeMethod( p.sender, p.slot, Q_ARG( QDomDocument, document ), Q_ARG( bool, b ) );
				}
			}
		default:
			break;
		}
		// delete temporary file
		delete p.file;
		// remove request from list
		mRequests.remove( i );
	}
}
//
void pHttpRequest::login( QObject* o, const char* s, QString l, QString p )
{
	// if no user infos, get them from settings
	if ( l.isEmpty() || p.isEmpty() )
	{
		FreeAccount fa = misc::identify();
		l = fa.Login;
		p = fa.Password;
	}
	// send login request
	mIdLogin = requestUrl( o, s, QString( "%1?login=%2&pass=%3" ).arg( freeLoginUrl(), l, p ) );
}
