/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : dimanche 30 juillet 2006 - 12h24
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UIEmail.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIEmail.h"
#include "main.h"
#include "misc.h"
//
#include <QShowEvent>
#include <QHeaderView>
#include <QSettings>
//
const QString mEmailUrl = "https://subscribe.free.fr/accesgratuit/acces/newaccount.pl";
//
QPointer<UIEmail> UIEmail::mSelf = 0L;
//
UIEmail* UIEmail::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIEmail( p );
	return mSelf;
}
//
UIEmail::UIEmail( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	fixMacWidgets();
	// Action for the context menu
	QAction* aDelete = new QAction( "&Supprimer", twEmails );
	twEmails->addAction( aDelete );
	connect( aDelete, SIGNAL( triggered() ), this, SLOT( removeEmailAccount() ) );
	// loads mail accounts
	loadMailAccounts();
}
//
void UIEmail::showEvent( QShowEvent* e )
{
	fgDialog::showEvent( e );
	//
	setWidgetTransparent( twEmails, backgroundPixmap() );
	foreach ( QLineEdit* e, findChildren<QLineEdit*>() )
		setWidgetTransparent( e, backgroundPixmap() );
	//
	twEmails->setItemHidden( twEmails->headerItem(), true );
	twEmails->setColumnWidth( 0, 234 );
	twEmails->setColumnWidth( 1, 25 );
}
//
void UIEmail::loadMailAccounts()
{
	twEmails->clear();
	QStringList l = misc::emailAccounts();
	QTreeWidgetItem* i;
	foreach ( QString s, l )
	{
		i = new QTreeWidgetItem( twEmails );
		i->setText( 0, s );
		i->setIcon( 1, QIcon( ":/email/Pictures/email/btn-goemail01.png" ) );
	}
}
//
void UIEmail::removeEmailAccount()
{
	if ( !twEmails->currentItem() )
		return;
	misc::emailRemoveAccount( twEmails->currentItem()->text( 0 ) );
	loadMailAccounts();
}
//
void UIEmail::on_twEmails_itemClicked( QTreeWidgetItem* i, int c )
{
	if ( i && c == 1 )
		misc::openMailBox( i->text( 0 ) );
}
//
void UIEmail::on_fgAdd_clicked()
{
	if ( leAddEmail->text().isEmpty() || leAddPassword->text().isEmpty() )
	{
		UIMAIN->alert( "Veuillez remplir votre adresse email et votre mot de passe." );
		return;
	}
	// checking email address
	if ( leAddEmail->text().indexOf( "@" ) == -1 )
	{
		leAddEmail->setText( leAddEmail->text().append( "@free.fr" ) );
		//lePop->setText( "pop3.free.fr" );
	}
	// check email length
	if ( leAddPassword->text().length() < 6 || leAddPassword->text().length() > 8 ) 
	{
		UIMAIN->alert( QString::fromUtf8( "Votre mot de passe doit contenir de 6 à 8 caractères." ) );
		return;
	}
	/*
	if ( leAddEmail->text().indexOf( "@free.fr", Qt::CaseInsensitive ) != -1 )
		lePop->setText( "pop3.free.fr" );
	*/
	misc::emailAddAccount( leAddEmail->text(), leAddPassword->text(), "pop3.free.fr" );
	loadMailAccounts();
}
//
void UIEmail::on_fgCreate_clicked()
{
	// Check Informations...
	if ( leCreateEmail->text().isEmpty() || leCreatePassword->text().isEmpty() ) 
	{
		UIMAIN->alert( "Veuillez rentrer un identifiant ainsi qu'un mot de passe." );
		return;
	}
	else if ( leCreatePassword->text().length() < 6 ) 
	{
		UIMAIN->alert( QString::fromUtf8( "Votre mot de passe est trop court, la taille de celui-ci doit être de 6 caractères minimum." ) );
		return;
	}
	else if ( leCreatePassword->text() != leCreateConfirme->text() ) 
	{
		UIMAIN->alert( "Les mots de passe ne sont pas identiques." );
		return;
	}
	REQUESTER->login( this, "readLogin" );
}
//
void UIEmail::readLogin( QSettings*, bool error )
{
	if ( error )
	{
		UIMAIN->alert( "Impossible de vous identifier sur free, annulation" );
		return;
	}
	// login
	QString login = leCreateEmail->text();
	if ( login.contains( "@" ) )
		login = login.left( login.indexOf( "@" ) );
	// url
	QString u = QString( mEmailUrl );
	// request
	QByteArray donnees =	QByteArray ("boite=")
							.append( login )
							.append( "&boitepass=" )
							.append( leCreatePassword->text() )
							.append( "&boitepass2=" )
							.append( leCreateConfirme->text() )
							.append( "&newmail=Etape+suivante" );
	
	REQUESTER->requestUrl( this, "readCreateEmail", u, "POST", pHttpRequest::rtByteArray, donnees );
}
//
void UIEmail::readCreateEmail( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( QString::fromUtf8( "Impossible de créer votre boîte mail." ) );
		return;
	}
	
	// parse
	QString ba = QString::fromLatin1( buffer );
	
	QString key = "<div  id=\"content\" class=\"internet\">";
	int pos = ba.indexOf( key );
	ba = ba.mid( pos );
	
	key = "<div class=\"clearer\">";
	pos = ba.indexOf( key );
	ba = ba.left( pos ).trimmed();
	
	// get result
	QString r;
	QRegExp rx( "<p>((.*))</p>" );
	rx.setMinimal( true );
	pos = 0;
	if ( ( pos = rx.indexIn( ba.simplified() ) ) != -1 )
		r = rx.cap( 1 ).simplified();
	
	if ( r.contains( "<br>" ) )
		r = r.split( "<br>" ).value( 0 ).trimmed();
	if ( !r.isEmpty() )
		r.append( "." );
	
	// give message according to result
	if ( r.startsWith( "Vous ne pouvez utiliser que des" ) ||
		r.startsWith( "Le mot de passe" ) ||
		r.startsWith( "La boite aux lettres" ) )
	{
		UIMAIN->alert( r, true, true );
		
	}
	else if ( r.startsWith( "La nouvelle boite aux lettres" ) )
	{
		//UIMAIN->alert( r );
		if ( !leCreateEmail->text().contains( "@free.fr", Qt::CaseInsensitive ) )
			leCreateEmail->setText( leCreateEmail->text().append( "@free.fr" ) );
		UIMAIN->alert( QString::fromUtf8( "Votre nouveau compte email a été créé.\n"
			"Identifiant : %1\nMot de passe : %2" ).arg( leCreateEmail->text(), leCreatePassword->text() ), true );
		misc::emailAddAccount( leCreateEmail->text(), leCreatePassword->text(), "pop3.free.fr" );
		loadMailAccounts();
	}
}
