/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : mardi 01 aot 2006 - 19h23
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UINotify.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UINotify.h"
#include "main.h"
#include "Options.h"
#include "misc.h"
#include "UIPublicity.h"
#include "ui/UIPhone.h"
#include "ThreadFreeGoInformations.h"

#include <QMouseEvent>
#include <QDesktopWidget>
#include <QTimer>
#include <QFile>
#include <QDateTime>

/*******************************
*		UINotify Class
*******************************/
QPointer<UINotify> UINotify::mLastUINotify = 0L;

UINotify::UINotify( NotifyType t, QWidget* p )
	: fgDialog( p ? p : UIMAIN )
{
	setupUi( this );
	fixMacWidgets();
	setAttribute( Qt::WA_DeleteOnClose );
	//setFocus( Qt::NoFocus );
	QPoint mPosition;
	setType( t );

	if ( !mLastUINotify )
	{
		mPosition = QPoint( QApplication::desktop()->width() -width(), QApplication::desktop()->height() -height() -34 );
		//position = Options::current()->value( "FreeGo/Notifications/LastPosition", defaultPosition ).toPoint();
	}
	else // stack them up (or down) if multiple notification windows are visible at the same time
	{
		mPosition = mLastUINotify->geometry().topLeft();
		if ( mPosition.y() < QApplication::desktop()->height() / 2 )
			mPosition.ry() += height();
		else
			mPosition.ry() -= height();
	}

	// After going down too much, start again from the top
	if ( mPosition.y() +height() > QApplication::desktop()->height() )
		mPosition.ry() = 0;

	// After going up too much, start again from the bottom
	if ( mPosition.y() -height() < 0 )
		mPosition.ry() = QApplication::desktop()->height() -height();

	move( mPosition );
	mLastUINotify = this;
}

void UINotify::moveEvent( QMoveEvent* e )
{
	fgDialog::moveEvent( e );
	Options::current()->setValue( "FreeGo/Notifications/LastPosition", geometry().topLeft() );
}

void UINotify::setType( UINotify::NotifyType t )
{
	QString mBackground = ":/notify/Pictures/notify/";
	QString mCloseNormal = ":/boutons/Pictures/boutons/btn-fermer01.png";
	QString mClosePushed = ":/boutons/Pictures/boutons/btn-fermer01.png";
	switch ( t )
	{
	case ntAlert:
		mBackground.append( "team.png" );
		break;
	case ntNewEmails:
		mBackground.append( "emails.png" );
		break;
	case ntProgramUpdate:
		mBackground.append( "team.png" );
		break;
	case ntProgramUpdating:
		return;
		break;
	case ntFreeBoxUpdate:
		mBackground.append( "freebox.png" );
		break;
	case ntFreeActualityUpdate:
		mBackground.append( "actuality.png" );
		break;
	case ntTeamMessageUpdate:
		mBackground.append( "team.png" );
		break;
	case ntTelephonyMessage:
		mBackground.append( "message.png" );
		break;
	default:
		return;
		break;
	}
	mType = t;
	setPixmaps( mBackground, mCloseNormal, mClosePushed );
}

UINotify::NotifyType UINotify::type() const
{
	return mType;
}

void UINotify::setPixmaps( const QString& b, const QString& n, const QString& p )
{
	QPixmap pb( b );
	if ( !pb.isNull() )
	{
		setBackgroundPixmap( pb );
		setWidgetTransparent( tbInformations, pb );
	}
	else
		qWarning( "Background pixmap not found: %s", qPrintable( b ) );

	QPixmap pn( n );
	if ( !pn.isNull() )
		fgClose->setButtonNormalPixmap( pn );
	else
		qWarning( "Close pixmap not found: %s", qPrintable( n ) );

	QPixmap pp( p );
	if ( !pp.isNull() )
		fgClose->setButtonPushedPixmap( pp );
	else
		qWarning( "Button pushed pixmap not found: %s", qPrintable( p ) );
}

void UINotify::on_tbInformations_anchorClicked( const QUrl& u )
{
	QString m;
	switch ( mType )
	{
	case ntNewEmails:
		misc::openMailBox( u.toString().mid( 1 ) );
		break;
	case ntProgramUpdate:
		Notify::self()->programUpdating( -1, QDateTime(), u.toString() );
		break;
	case ntTelephonyMessage:
		UIPhone::self( UIMAIN )->swRight->setCurrentIndex( 1 );
		UIPhone::self( UIMAIN )->show();
		break;
	default:
		if ( !misc::openUrl( u ) )
			UIMAIN->alert( QString::fromUtf8( "Impossible de trouver l'application associée à ce schéma" ) );
		break;
	}
	close();
}

/*******************************
*		Notify Class
*******************************/
QPointer<Notify> Notify::mSelf = 0L;

Notify* Notify::self( QObject* p )
{
	if ( !mSelf )
		mSelf = new Notify( p );
	return mSelf;
}

Notify::Notify( QObject* p )
	: QObject( p )
{
}

void Notify::alert( const QString& s )
{
	UINotify* n = new UINotify( UINotify::ntAlert );
	n->tbInformations->setHtml( s );
	QTimer::singleShot( 15000, n, SLOT( close() ) );
	n->show();
}

void Notify::newEmails( const QString& l, int i )
{
	UINotify* n = new UINotify( UINotify::ntNewEmails );
	QString s = QString( "<font color=\"#000000\"><a href=#%1>%2</a> : %3 message(s) non lu(s).</font>" ).arg( l, l ).arg( i );
	s = QString::fromLocal8Bit( misc::processLinks( s.toLocal8Bit(), "#0000FF" ) );
	n->tbInformations->setHtml( s );
	QTimer::singleShot( 15000, n, SLOT( close() ) );
	n->show();
}

void Notify::programUpdate( int, const QDateTime&, const QString& s )
{
	UINotify* n = new UINotify( UINotify::ntProgramUpdate );
	QString m = QString( "<font color=\"#0000FF\">%1</font>" ).arg( s );
	m = QString::fromLocal8Bit( misc::processLinks( m.toLocal8Bit() ) );
	n->tbInformations->setHtml( m );
	QTimer::singleShot( 15000, n, SLOT( close() ) );
	n->show();
}

void Notify::programUpdating( int, const QDateTime&, const QString& s )
{
	QString m = s;
#if defined( Q_OS_WIN32 )
	m = m.arg( "_win32.exe" );
#elif defined( Q_OS_MAC )
	m = m.arg( "_mac" );
#elif defined( Q_OS_UNIX )
	m = m.arg( "_unix" );
#endif
	REQUESTER->requestUrl( UIMAIN, "updateProgram", m, "GET", pHttpRequest::rtFile );
}

void Notify::freeboxUpdate( int, const QDateTime&, const QString& s )
{
	UINotify* n = new UINotify( UINotify::ntFreeBoxUpdate );
	QString m = QString( "<font color=\"#000000\">%1</font>" ).arg( s );
	m = QString::fromLocal8Bit( misc::processLinks( m.toLocal8Bit() ) );
	n->tbInformations->setHtml( m );
	QTimer::singleShot( 15000, n, SLOT( close() ) );
	n->show();
}

void Notify::freeActualityUpdate( int, const QDateTime&, const QString& s )
{
	UINotify* n = new UINotify( UINotify::ntFreeActualityUpdate );
	QString m = QString( "<font color=\"#000000\">%1</font>" ).arg( s );
	m = QString::fromLocal8Bit( misc::processLinks( m.toLocal8Bit() ) );
	n->tbInformations->setHtml( m );
	QTimer::singleShot( 15000, n, SLOT( close() ) );
	n->show();
}

void Notify::teamMessageUpdate( int, const QDateTime&, const QString& s )
{
	UINotify* n = new UINotify( UINotify::ntTeamMessageUpdate );
	QString m = QString( "<font color=\"#000000\">%1</font>" ).arg( s );
	m = QString::fromLocal8Bit( misc::processLinks( m.toLocal8Bit() ) );
	n->tbInformations->setHtml( m );
	QTimer::singleShot( 15000, n, SLOT( close() ) );
	n->show();
}

void Notify::freegoAnnonce( int, const QDateTime&, const QString& s, const QString& f )
{
	QWidget* publigo = UIPublicity::self( f, s, UIMAIN );
	publigo->setWindowModality( Qt::ApplicationModal );
	connect( publigo, SIGNAL( hidden() ), ThreadFreeGoInformations::self(), SLOT( setPubligoShow() ) );
	publigo->show();
}

void Notify::newMessages( int count )
{
	if ( count > 0 )
	{
		UINotify* n = new UINotify( UINotify::ntTelephonyMessage );
		QString m = count > 0 ? QString::fromUtf8( "Vous avez <font color=\"red\">%1</font> message(s) sur votre répondeur.\nCliquez <a href=\"fakeUrl\">ici</a> pour les lire." ).arg( count ) : QString::fromUtf8( "Pas de nouveaux messages sur votre répondeur." );
		m = QString::fromLocal8Bit( misc::processLinks( m.toLocal8Bit(), "#000000" ) );
		n->tbInformations->setHtml( m );
		QTimer::singleShot( 15000, n, SLOT( close() ) );
		n->show();
	}
}
