#include "UIRooterWidget.h"
#include "fgDialog.h"

#include <QVBoxLayout>
#include <QScrollBar>

UIRooterWidget::UIRooterWidget( QWidget* parent )
	: QWidget( parent )
{
	mType = Undefined;
	setupUi( this );
	fgDialog::fixMacWidgets( this );
	mVLayout = new QVBoxLayout( wContents );
	mVLayout->setMargin( 5 );
	mVLayout->setSpacing( 3 );
}

void UIRooterWidget::setType( Type type )
{
	// clear
	for ( int i = 0; i < hlayout->count(); i++ )
		delete hlayout->itemAt( i )->widget();
	for ( int i = 0; i < mVLayout->count(); i++ )
		delete mVLayout->itemAt( i )->widget();
	// update gui
	mType = type;
	QLabel* label = 0;
	switch ( mType )
	{
		case Ports:
			label = new QLabel( "Port", this );
			label->setFixedWidth( 65 );
			label->setAlignment( Qt::AlignCenter );
			hlayout->addWidget( label );
			
			label = new QLabel( "Protocole", this );
			label->setFixedWidth( 75 );
			label->setAlignment( Qt::AlignCenter );
			hlayout->addWidget( label );
			
			label = new QLabel( "Destination", this );
			label->setFixedWidth( 140 );
			label->setAlignment( Qt::AlignCenter );
			hlayout->addWidget( label );
			
			label = new QLabel( "Port", this );
			label->setIndent( 30 );
			hlayout->addWidget( label );
			break;
		case Plages:
			label = new QLabel( QString::fromUtf8( "Début" ), this );
			label->setFixedWidth( 65 );
			label->setAlignment( Qt::AlignCenter );
			hlayout->addWidget( label );
			
			label = new QLabel( "Fin", this );
			label->setFixedWidth( 75 );
			label->setAlignment( Qt::AlignCenter );
			hlayout->addWidget( label );
			
			label = new QLabel( "Protocole", this );
			label->setFixedWidth( 73 );
			label->setAlignment( Qt::AlignCenter );
			hlayout->addWidget( label );
			
			label = new QLabel( "Destination", this );
			label->setIndent( 45 );
			hlayout->addWidget( label );
			break;
		case Macs:
			label = new QLabel( "IP", this );
			label->setFixedWidth( 145 );
			label->setAlignment( Qt::AlignCenter );
			hlayout->addWidget( label );
			
			label = new QLabel( "Adresse MAC", this );
			label->setIndent( 85 );
			hlayout->addWidget( label );
			break;
		default:
			break;
	}
	for ( int i = 0; i < 10; i++ )
		addLine();
	updateButtonsVisibility();
}

QStringList UIRooterWidget::datas() const
{
	QStringList datas;
	for ( int i = 0; i < mVLayout->count(); i++ )
	{
		QLayoutItem* item = mVLayout->itemAt( i );
		if ( AbstractRooterWidget* arw = qobject_cast<AbstractRooterWidget*>( item->widget() ) )
		{
			if ( arw->isValid() )
				datas << arw->datas().join( "|" );
		}
	}
	return datas;
}

void UIRooterWidget::setIp3( const QString& value )
{
	for ( int i = 0; i < mVLayout->count(); i++ )
	{
		QLayoutItem* item = mVLayout->itemAt( i );
		if ( AbstractRooterWidget* arw = qobject_cast<AbstractRooterWidget*>( item->widget() ) )
			arw->setIp( value, 3 );
	}
}

void UIRooterWidget::setDatas( const QStringList& datas )
{
	// create new lines if needed
	const int count = datas.count();
	if ( mVLayout->count() < count )
	{
		while ( mVLayout->count() != count )
			addLine();
		updateButtonsVisibility();
	}
	// fill datas
	for ( int i = 0; i < mVLayout->count(); i++ )
	{
		QLayoutItem* item = mVLayout->itemAt( i );
		if ( AbstractRooterWidget* arw = qobject_cast<AbstractRooterWidget*>( item->widget() ) )
			arw->setDatas( datas.value( i ).split( "|", QString::SkipEmptyParts ) );
	}
}

QString UIRooterWidget::ip3()
{
	if ( mVLayout->count() > 0 )
	{
		QLayoutItem* item = mVLayout->itemAt( 0 );
		if ( AbstractRooterWidget* arw = qobject_cast<AbstractRooterWidget*>( item->widget() ) )
			return arw->ip( 3 );
	}
	return QString::null;
}

void UIRooterWidget::updateButtonsVisibility()
{
	const int count = mVLayout->count();
	for ( int i = 0; i < count; i++ )
	{
		QLayoutItem* item = mVLayout->itemAt( i );
		AbstractRooterWidget* w = qobject_cast<AbstractRooterWidget*>( item->widget() );
		if ( w )
		{
			w->setRemoveLineVisible( count > 1 );
			w->setAddLineVisible( i == mVLayout->count() -1 );
		}
	}
}

void UIRooterWidget::addLine()
{
	AbstractRooterWidget* rw = 0;
	switch ( mType )
	{
		case Ports:
			rw = new UIPortsRedirection( wContents );
			break;
		case Plages:
			rw = new UIPlagesRedirection( wContents );
			break;
		case Macs:
			rw = new UIPermanentsDHCP( wContents );
			break;
		default:
			break;
	}
	if ( rw )
	{
		fgDialog::fixMacWidgets( rw );
		rw->setIp( ip3(), 3 );
		mVLayout->addWidget( rw );
		connect( rw, SIGNAL( addLineRequested() ), this, SLOT( addLine() ) );
		connect( rw, SIGNAL( removeLineRequested() ), this, SLOT( removeLine() ) );
	}
	if ( sender() )
		updateButtonsVisibility();
}

void UIRooterWidget::removeLine()
{
	if ( !sender() )
		return;
	mVLayout->removeWidget( qobject_cast<QWidget*>( sender() ) );
	sender()->deleteLater();
	updateButtonsVisibility();
}
