/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : jeudi 27 juillet 2006 - 18h57
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UISubscribe.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UISubscribe.h"
#include "main.h"
#include "UILineFeature.h"
#include "misc.h"

QPointer<UISubscribe> UISubscribe::mSelf = 0L;

UISubscribe* UISubscribe::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UISubscribe( p );
	return mSelf;
}

UISubscribe::UISubscribe( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	fixMacWidgets();
	REQUESTER->login( this, "readLogin" );
}

void UISubscribe::readLogin( QSettings*, bool b )
{
	if ( b )
	{
		UIMAIN->alert( "Impossible de vous identifier sur free, annulation" );
		deleteLater();
		return;
	}
	REQUESTER->requestUrl( this, "readSubscribe", REQUESTER->freeBaseUrl().append( "commande.php" ), "GET", pHttpRequest::rtByteArray );
}

void UISubscribe::addLine( const QString& text, const QString& pixmap, Qt::GlobalColor color )
{
	QList<QLabel*> labels;
	QLabel* label;
	
	// pixmap
	if ( !pixmap.isEmpty() )
	{
		label = new QLabel( this );
		label->setPixmap( QPixmap( pixmap ) );
		label->setAlignment( Qt::AlignTop | Qt::AlignLeft );
		label->setSizePolicy( QSizePolicy( QSizePolicy::Maximum, QSizePolicy::Preferred ) );
		labels << label;
	}
	
	// text
	label = new QLabel( text, this );
	label->setWordWrap( true );
	label->setTextFormat( Qt::RichText );
	label->setAlignment( Qt::AlignTop | Qt::AlignHCenter );
	if ( !pixmap.isEmpty() )
		label->setAlignment( Qt::AlignTop | Qt::AlignLeft );
	QPalette palette( label->palette() );
	palette.setColor( label->foregroundRole(), color );
	QFont font( label->font() );
	if ( color == Qt::red )
		font.setBold( true );
	label->setFont( font );
	label->setPalette( palette );
	connect( label, SIGNAL( linkActivated( const QString& ) ), this, SLOT( linkActivated( const QString& ) ) );
	labels << label;
	
	if ( labels.count() == 1 )
		vlLayout->addWidget( labels.at( 0 ) );
	else
	{
		QHBoxLayout* hbLayout = new QHBoxLayout;
		hbLayout->setSpacing( 16 );
		foreach ( QLabel* l, labels )
			hbLayout->addWidget( l );
		vlLayout->addLayout( hbLayout );
	}
}

void UISubscribe::readSubscribe( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( QString::fromUtf8("Impossible de tlcharger le fichier suivi.pl") );
		return;
	}
	
	// parse
	QDomDocument e( "FreeLineData" );
	e.setContent( QString::fromLatin1( buffer ) );
	QDomElement root = e.firstChildElement( "suivi" );
	
	bool selectedFound = false;
	for(int i = 1;i < 7;i++)
	{
		QDomElement etape = root.firstChildElement( QString( "etape%1" ).arg( i ) );
		QString color = selectedFound ? QString( "gris" ) : QString( "bleu" );
		Qt::GlobalColor cColor = selectedFound ? Qt::black : Qt::black;
		
		if( etape.attribute( "selected" ) == "true" )
		{
			selectedFound = true;
			color = "red";
			cColor = Qt::red;
		}
		
		QString fn = QString( ":/suivi/Pictures/suivi/%1%2.png" ).arg( i ).arg( color );
		addLine( etape.text(), fn, cColor );
	}
	
	fixMacWidgets();
}

void UISubscribe::linkActivated( const QString& url )
{
	if ( url.contains( "suivi_techgrrr.pl" ) )
		UILineFeature::self()->show();
	else
	{
		QUrl u( url );
		if ( u.isRelative() )
			u.setPath( REQUESTER->freeBaseUrl().append( "suivi/" ) );
		misc::openUrl( u );
	}
}
