/********************************************************************************************************
* PROGRAM      : FreeGo4
* DATE - TIME  : dimanche 01 octobre 2006 - 16h01
* AUTHOR       : Azevedo Filipe aka Nox PasNox ( pasnox at gmail dot com )
* FILENAME     : UIWifiRooter.cpp
* LICENSE      : GPL
* COMMENTARY   : 
********************************************************************************************************/
#include "UIWifiRooter.h"
#include "main.h"
#include "misc.h"

#include <QSettings>
#include <QtXml>

#include <math.h>

QPointer<UIWifiRooter> UIWifiRooter::mSelf = 0L;

UIWifiRooter* UIWifiRooter::self( QWidget* p )
{
	if ( !mSelf )
		mSelf = new UIWifiRooter( p );
	return mSelf;
}

UIWifiRooter::UIWifiRooter( QWidget* p )
	: fgDialog( p )
{
	setupUi( this );
	rwPorts->setType( UIRooterWidget::Ports );
	rwPlages->setType( UIRooterWidget::Plages );
	rwDHCP->setType( UIRooterWidget::Macs );
	fixMacWidgets();
	
	// connections
	// menu
	QList<fgLabel*> buttons = findChildren<fgLabel*>( QRegExp( "fgMenu" ) );
	foreach ( fgLabel* button, buttons )
		connect( button, SIGNAL( clicked( const int ) ), this, SLOT( fgMenu_clicked( const int ) ) );
	// ip
	connect( leIpFreebox3, SIGNAL( textChanged( const QString& ) ), this, SLOT( setIp3( const QString& ) ) );
	// rooter
	connect( fgSaveRooter1, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	connect( fgSaveRooter2, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	connect( fgSaveRooter3, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	connect( fgSaveRooter4, SIGNAL( clicked() ), this, SLOT( fgSaveRooter() ) );
	// wifi
	connect( fgSaveWifi1, SIGNAL( clicked() ), this, SLOT( fgSaveWifi() ) );
	connect( fgSaveWifi2, SIGNAL( clicked() ), this, SLOT( fgSaveWifi() ) );
	
	// login
	REQUESTER->login( this, "readLogin" );
}

void UIWifiRooter::fgMenu_clicked( int i )
{
	QList<fgLabel*> buttons = findChildren<fgLabel*>( QRegExp( "fgMenu" ) );
	foreach ( fgLabel* button, buttons )
	{
		button->setToggled( button->data().toInt() == i );
		if ( sender() && button->data().toInt() == i )
			button->setToggled( false );
	}
	swPages->setCurrentIndex( i );
}

void UIWifiRooter::setIp3( const QString& value )
{
	leDebutDHCP3->setText( value );
	leIpDmz3->setText( value );
	leFinDHCP3->setText( value );
	leIpFreeplayer3->setText( value );
	rwPorts->setIp3( value );
	rwPlages->setIp3( value );
	rwDHCP->setIp3( value );
}

void UIWifiRooter::readLogin( QSettings*, bool error )
{
	if ( error )
	{
		UIMAIN->alert( "Impossible de vous identifier sur free, annulation" );
		deleteLater();
		return;
	}
	requestRooter( QByteArray(), false );
	requestWifi( QByteArray(), false );
	fgMenu_clicked( 0 );
}

void UIWifiRooter::requestRooter( const QByteArray& buffer, bool error )
{
	// check buffer
	if ( !buffer.isEmpty() && !buffer.contains( "Valeurs mises &agrave; jour avec succ&egrave;s" ) )
		error = true;
	if ( error )
	{
		QString ba = QString::fromUtf8( buffer );
		int pos = ba.indexOf( "fbxcfgwww_result" );
		pos = ba.indexOf( "<p>", pos ) +3;
		int end = ba.indexOf( "</p>", pos );
		ba = ba.mid( pos, end -pos ).simplified();
		
		QString msg = "Une erreur est survenue lors de la modification (Router)";
		if ( !ba.isEmpty() )
			msg.append( "\nErreur Free: " +ba );
		UIMAIN->alert( msg );
		pConfigRooter->setEnabled( true );
		pPortsRedirect->setEnabled( true );
		pPortsPlage->setEnabled( true );
		pBaudsDHCP->setEnabled( true );
		return;
	}
	// request rooter configuration
	REQUESTER->requestUrl( this, "readRooter", REQUESTER->freeBaseUrl().append( "routeur.php" ), "GET", pHttpRequest::rtByteArray );
	// disable pages
	pConfigRooter->setEnabled( false );
	pPortsRedirect->setEnabled( false );
	pPortsPlage->setEnabled( false );
	pBaudsDHCP->setEnabled( false );
}

void UIWifiRooter::readRooter( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( QString::fromUtf8("Une erreur est survenue lors de la rcuperation de la configuration Router") );
		return;
	}
	
	QDomDocument xmldoc("Free");
	xmldoc.setContent( buffer );
	
	QStringList ports_datas, plages_datas, macs_datas;
	
	QDomNode root = xmldoc.firstChild();
	QDomElement rootElem = root.toElement();
	cbActivation->setToggled( rootElem.attribute( "rt_enable" ) == "true" );
	leIpFreebox3->setText( rootElem.attribute( "lan_pfx" ) );
	leIpFreebox4->setText( rootElem.attribute( "lan_ip" ) );
	cbDHCPActivation->setToggled( rootElem.attribute( "dhcp_active" ) == "true" );
	cbUPnP->setToggled( rootElem.attribute( "upnp_active" ) == "true" );
	leDebutDHCP4->setText( rootElem.attribute( "dhcp_start" ) );
	leFinDHCP4->setText( rootElem.attribute( "dhcp_end" ) );
	leIpDmz4->setText( rootElem.attribute( "dmz_host" ) );
	leIpFreeplayer4->setText( rootElem.attribute( "freeplayer" ) );
	cbReponsePing->setToggled( rootElem.attribute( "answer_ping" ) == "true" );
	cbWOL->setToggled( rootElem.attribute( "proxy_wol" ) == "true" );
	
	QDomNode n = root.firstChild();
	while(!n.isNull())
	{
		QDomElement e = n.toElement();
		
		if( e.tagName() == "redir" )
		{
			ports_datas << QString( "%1|%2|%3|%4" )
						 .arg( e.attribute( "srcport" ) )
						 .arg( e.attribute( "proto" ) )
						 .arg( e.attribute( "dst" ) )
						 .arg( e.attribute( "dstport" ) );
		}
		else if( e.tagName() == "redirp" )
		{
			plages_datas << QString( "%1|%2|%3|%4" )
						 .arg( e.attribute( "port1" ) )
						 .arg( e.attribute( "port2" ) )
						 .arg( e.attribute( "proto" ) )
						 .arg( e.attribute( "dst" ) );
		}
		else if( e.tagName() == "dhcp" )
		{
			macs_datas << QString( "%1|%2" )
						 .arg( e.attribute( "ip" ) )
						 .arg( e.attribute( "mac" ) );
		}
		
		n = n.nextSibling();
	}
	
	rwPorts->setDatas( ports_datas );
	rwPlages->setDatas( plages_datas );
	rwDHCP->setDatas( macs_datas );
	
	// enable pages
	pConfigRooter->setEnabled( true );
	pPortsRedirect->setEnabled( true );
	pPortsPlage->setEnabled( true );
	pBaudsDHCP->setEnabled( true );
}

void UIWifiRooter::fgSaveRooter()
{
	// post change
	QString mPost = REQUESTER->freeSSLBaseUrl().append( "admin/fbxcfg/fbxcfg.pl?tpl=router&action=update&Envoyer" );
	mPost += QString( "&router_active=%1" ).arg( cbActivation->toggled() ? 1 : 0 );
	mPost += QString( "&lan_pfx=%1" ).arg( leIpFreebox3->text() );
	mPost += QString( "&lan_pfx_copy=%1" ).arg( leIpFreebox3->text() );
	mPost += QString( "&lan_ip=%1" ).arg( leIpFreebox4->text() );
	mPost += QString( "&dhcp_active=%1" ).arg( cbDHCPActivation->toggled() ? 1 : 0 );
	mPost += QString( "&dhcp_start=%1" ).arg( leDebutDHCP4->text() );
	mPost += QString( "&dhcp_end=%1" ).arg( leFinDHCP4->text() );
	mPost += QString( "&dmz_host=%1" ).arg( leIpDmz4->text() );
	mPost += QString( "&freeplayer=%1" ).arg( leIpFreeplayer4->text() );
	mPost += QString( "&answer_ping=%1" ).arg( cbReponsePing->toggled() ? 1 : 0 );
	mPost += QString( "&proxy_wol=%1" ).arg( cbWOL->toggled() ? 1 : 0 );
	mPost += QString( "&upnp_active=%1" ).arg( cbUPnP->toggled() ? 1 : 0 );
	// ports
	const QStringList ports_datas = rwPorts->datas();
	for ( int i = 0; i < ports_datas.count(); i++ )
	{
		const QStringList datas = ports_datas.at( i ).split( "|", QString::SkipEmptyParts );
		mPost += QString( "&redir_port_%1=%2" ).arg( i ).arg( datas.value( 0 ) );
		mPost += QString( "&redir_proto_%1=%2" ).arg( i ).arg( datas.value( 1 ) );
		mPost += QString( "&redir_dest_%1=%2" ).arg( i ).arg( datas.value( 2 ) );
		mPost += QString( "&redir_dest_port_%1=%2" ).arg( i ).arg( datas.value( 3 ) );
	}
	// plages
	const QStringList plages_datas = rwPlages->datas();
	for ( int i = 0; i < plages_datas.count(); i++ )
	{
		const QStringList datas = plages_datas.at( i ).split( "|", QString::SkipEmptyParts );
		mPost += QString( "&mredir_port_%1=%2" ).arg( i ).arg( datas.value( 0 ) );
		mPost += QString( "&mredir_dest_port_%1=%2" ).arg( i ).arg( datas.value( 1 ) );
		mPost += QString( "&mredir_proto_%1=%2" ).arg( i ).arg( datas.value( 2 ) );
		mPost += QString( "&mredir_dest_%1=%2" ).arg( i ).arg( datas.value( 3 ) );
	}
	// plages
	const QStringList macs_datas = rwDHCP->datas();
	for ( int i = 0; i < macs_datas.count(); i++ )
	{
		const QStringList datas = macs_datas.at( i ).split( "|", QString::SkipEmptyParts );
		mPost += QString( "&lease_ip_%1=%2" ).arg( i ).arg( datas.value( 0 ) );
		mPost += QString( "&lease_mac_%1=%2" ).arg( i ).arg( datas.value( 1 ) );
	}

	// send request
	REQUESTER->requestUrl( this, "requestRooter", mPost, "GET", pHttpRequest::rtByteArray );
	
	// disable pages
	pConfigRooter->setEnabled( false );
	pPortsRedirect->setEnabled( false );
	pPortsPlage->setEnabled( false );
	pBaudsDHCP->setEnabled( false );
}

void UIWifiRooter::requestWifi( const QByteArray& buffer, bool error )
{
	// check buffer
	if ( !buffer.isEmpty() && !buffer.contains( "Valeurs mises &agrave; jour avec succ&egrave;s" ) )
		error = true;
	if ( error )
	{
		QString ba = QString::fromUtf8( buffer );
		int pos = ba.indexOf( "fbxcfgwww_result" );
		pos = ba.indexOf( "<p>", pos ) +3;
		int end = ba.indexOf( "</p>", pos );
		ba = ba.mid( pos, end -pos ).simplified();
		
		QString msg = "Une erreur est survenue lors de la modification (Wifi)";
		if ( !ba.isEmpty() )
			msg.append( "\nErreur Free: " +ba );
		UIMAIN->alert( msg );
		pConfigWifi->setEnabled( true );
		pWifiSecurity->setEnabled( true );
		return;
	}
	// request wifi configuration
	REQUESTER->requestUrl( this, "readWifi", REQUESTER->freeBaseUrl().append( "wifi.php" ), "GET", pHttpRequest::rtByteArray );
	pConfigWifi->setEnabled( false );
	pWifiSecurity->setEnabled( false );
}

void UIWifiRooter::readWifi( const QByteArray& buffer, bool error )
{
	if ( error )
	{
		UIMAIN->alert( QString::fromUtf8("Une erreur est survenue lors de la rcuperation de la configuration Wifi") );
		return;
	}
	
	// parse
	QDomDocument xmldoc("Free");
	xmldoc.setContent( buffer );
	
	QDomNode root = xmldoc.firstChild();
	QDomNode n = root.firstChild();
	QDomElement e = n.toElement();

	cbActivationService->setToggled( e.attribute("wifi_active") == "true" );
	cbAutomaticCanal->setToggled( e.attribute("wifi_channel_auto") == "true" );
	leNomReseau->setText( e.attribute("wifi_ssid") );
	cbHiddenNetwork->setToggled( e.attribute("wifi_ssid_hide") == "true" );
	
	QString value = e.attribute("wifi_key_type");
	if ( value == "1" )
		rbWep->setChecked( true );
	else if ( value == "3" )
		rbWpaTkip->setChecked( true );
	else if ( value == "4" )
		rbWpaAesCcmp->setChecked( true );
	else if ( value == "2" )
		rbWpaTkipAes->setChecked( true );
	
	leClef->setText( e.attribute("wifi_key") );
	
	cbCanal->setCurrentIndex( e.attribute("wifi_channel").toInt() - 1 );
	
	pConfigWifi->setEnabled( true );
	pWifiSecurity->setEnabled( true );
	
	cbEteindreModule->setToggled( root.toElement().attribute("wifi_disable_radio") == "true" );
}

void UIWifiRooter::on_fgGenerateKey_clicked()
// wep/wpa key generation, algo stolen from free.fr javascript script
{
	int wifiType = -1;
	if ( rbWep->isChecked() )
		wifiType = 1;
	else if ( rbWpaTkip->isChecked() )
		wifiType = 3;
	else if ( rbWpaAesCcmp->isChecked() )
		wifiType = 4;
	else if ( rbWpaTkipAes->isChecked() )
		wifiType = 2;
	
	QString chars;
	int len;
	switch ( wifiType )
	{
		case 1: // wep
			chars = "0123456789ABCDEF";
			len = 10; // 10 or 26
			break;
		case 2: // wpa
		case 3:
		case 4:
			chars = " abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890_\\-;.?,:!%*^&~{}()|@=+<>";
			len = 63;
			break;
		default:
			return;
			break;
	}
	
	QString key;
	for ( int i = 0; i < len; i++ )
	{
		int rnd = qrand() %( chars.length() );
		key += chars[ rnd ];
	}
	
	leClef->setText( key );
}

void UIWifiRooter::fgSaveWifi()
{
	if ( rbWep->isChecked() )
	{
		// wep
		if ( !QRegExp( "[A-F|0-9]{10}" ).exactMatch( leClef->text() ) && !QRegExp( "[A-F|0-9]{26}" ).exactMatch( leClef->text() ) )
		{
			UIMAIN->alert( "Votre clef WEP n'est pas valide." );
			return;
		}
	}
	else
	{
		// wpa
		if ( !QRegExp( ".{8,63}" ).exactMatch( leClef->text() ) )
		{
			UIMAIN->alert( "Votre clef WPA n'est pas valide." );
			return;
		}
	}
	
	// get wifi type
	int wifiType = -1;
	if ( rbWep->isChecked() )
		wifiType = 1;
	else if ( rbWpaTkip->isChecked() )
		wifiType = 3;
	else if ( rbWpaAesCcmp->isChecked() )
		wifiType = 4;
	else if ( rbWpaTkipAes->isChecked() )
		wifiType = 2;
	
	// check wifi type
	if ( wifiType == -1 )
	{
		UIMAIN->alert( "Vous devez choisir un type de protection." );
		return;
	}
	
	// post change
	QString mPost = REQUESTER->freeSSLBaseUrl().append( "admin/fbxcfg/fbxcfg.pl?tpl=wifi&action=update&Envoyer" );
	mPost += QString( "&wifi_disable_radio=%8&wifi_active=%1&wifi_channel=%2&wifi_ssid=%3&wifi_key_type=%4&wifi_key=%5&wifi_channel_auto=%6&wifi_ssid_hide=%7" )
		.arg( cbActivationService->toggled() )
		.arg( cbCanal->currentIndex() +1 )
		.arg( leNomReseau->text() )
		.arg( wifiType )
		.arg( leClef->text() )
		.arg( cbAutomaticCanal->toggled() )
		.arg( cbHiddenNetwork->toggled() )
		.arg( cbEteindreModule->toggled() );
	REQUESTER->requestUrl( this, "requestWifi", mPost, "GET", pHttpRequest::rtByteArray );
	
	pConfigWifi->setEnabled( false );
	pWifiSecurity->setEnabled( false );
}
